# -*- coding:utf-8 -*-
'''This script takes a Titanium project resources directory and generates .ts files for strings deemed translatable by L(text)

At the same time, if the .ts contain translations, then strings.xml files are produced which can be used directly in titanium

'''



import sys,os
import re

from xml.dom import minidom

from xml.dom.minidom import getDOMImplementation


APPLICATIONNAME = "Oarbits"
TRFUNCTION = "oarbits.tr"
PATH = ''

LANGUAGES = ['fr','de','es']

TRFUNCTION_LENGTH = len(TRFUNCTION) + 2

def FindJS(path):

    jsFiles = []

    for root, dirs, files in os.walk(path):

        for file in files:

            if file.endswith('.js'):

                jsFiles.append(os.path.join(root,file))

                

    return jsFiles



    

def GenerateTs(files):

    messages=[]

    for file in files:

        messages.extend(SearchForL(file))

    

    tsFiles=[]

    for l in LANGUAGES: 

        print "Updating .ts file for",l

        messagesUpdate = UpdateMessages(messages,l)

        tsFiles.append(WriteTsFile(messagesUpdate,l))

    return tsFiles

    

def GenerateStrings(tsFiles):

    for file,l in tsFiles:

        messages = ExtractMessages(file)

        WriteStrings(messages,l)

    WriteStrings(messages,'en')

            

def SearchForL(file):

    relpath = os.path.relpath(file,PATH)

    messages = []

    #print "Searching in",relpath

    with open(file,'r') as f:

        lines = f.readlines()        

        sources = []

        

        import re

        pattern = re.compile(r'%s[(][\'\"][^\'\"]*[\'\"][)]' % TRFUNCTION)

        #trans = urlPattern.findall(text)

            

         

        for i, line in enumerate(lines):

            search = pattern.search(line)

            if search:

                source = search.group()[TRFUNCTION_LENGTH:-2] #Cuts off the enclosing tr function

                m = {'source':source,

                    'filename':relpath,

                    'line':str(i+1),

                    'type':'unfinished',

                    'translation':''}

                #print m

                

                if sources.count(source)>0:

                    print "Ignoring duplicate source text \"%s\"" % source

                    continue

                

                sources.append(source)

                messages.append(m)

       

    return messages

    

            

def WriteTsFile(messages,language):

    impl = getDOMImplementation()

    

    doctype = impl.createDocumentType("TS", None, None)

    newdoc = impl.createDocument("TS", "TS", doctype)

    ts = newdoc.documentElement

    

    ts.setAttribute('version','2.0')

    ts.setAttribute('language',language)

    

    context = newdoc.createElement('context')

    name = newdoc.createElement('name')

    name.appendChild(newdoc.createTextNode(APPLICATIONNAME))

    context.appendChild(name)

    

    ts.appendChild(context)

    

    for message in messages:

        location = newdoc.createElement('location')

        source = newdoc.createElement('source')

        translation = newdoc.createElement('translation')

        

        location.setAttribute('filename',message['filename'])

        location.setAttribute('line',message['line'])

        

        source.appendChild(newdoc.createTextNode(message['source']))

        

        translation.setAttribute('type',message['type'])

        translation.appendChild(newdoc.createTextNode(message['translation'].encode('utf-8')))

        

        message = newdoc.createElement('message')

        

        message.appendChild(location)

        message.appendChild(source)

        message.appendChild(translation)

        

        context.appendChild(message)

    

    with open(os.path.join(OUTPUTPATH,'strings_%s.ts' % language),'w') as f:

        newdoc.writexml(f)

        print 'Wrote %s\n' % os.path.basename(f.name)

        return [f.name,language]

          

            

def UpdateMessages(messagesNew,language):

    #print "Update messages for %s" % language

    

    path = os.path.join(OUTPUTPATH,'strings_%s.ts' % language)

    

    #Check if .ts file exists

    if os.path.exists(path)==True:

        messagesOld = ExtractMessages(path)

    else: 

        messagesOld = []

    

    #extract messages from .ts file

    

    

    #compare to new messages

    messages = CompareMessages(messagesNew,messagesOld)

    

    return messages

    

def ExtractMessages(file):

    #print "Extract messages for %s" % os.path.relpath(file,PATH)

    extractedMessages = []
    import codecs

    with codecs.open(file,mode = 'r', encoding='utf-8') as f:
        dom = minidom.parseString(f.read().encode('utf-8'))

        messages = dom.getElementsByTagName("message")

        

        for message in messages:

            source = message.getElementsByTagName("source")[0].childNodes[0].nodeValue

            filename = message.getElementsByTagName("location")[0].getAttribute('filename')

            line = message.getElementsByTagName("location")[0].getAttribute('line')

            type = message.getElementsByTagName("translation")[0].getAttribute('type')

            

            translation = message.getElementsByTagName("translation")[0].childNodes

            if len(translation)!=0:

                translation = translation[0].nodeValue

            else:

                translation = ""

            

            m = {'source':source,

                    'filename':filename,

                    'line':line,

                    'type':type,

                    'translation':translation}

                    

            extractedMessages.append(m)

            #print m

            

    return extractedMessages



def CompareMessages(messagesNew,messagesOld):

    #Add new messages and update line numbers

    added = 0

    removed = 0

    

    for m in messagesNew:

        new = 1

        for i in range(0,len(messagesOld)):

            o = messagesOld[i]

            if (m['source']==o['source']): #and m['filename']==o['filename']):

                #Translation exists in old file

                new = 0 

        if new==1:

            #new translation, append to old

            messagesOld.append(m)

            #print "new string:",m['source']

            added+=1

            

    #Remove unused, if not translated

    j=0

    while j<len(messagesOld):

        o = messagesOld[j]

        unused = 1

        for i in range(0,len(messagesNew)):

            m = messagesNew[i]

            if (m['source']==o['source']): #and m['filename']==o['filename']):

                unused = 0 

        if unused==1 and o['translation']=='':

            #print "obselete string:",o['source']

            messagesOld.remove(messagesOld[j])

            removed+=1

            j=0

        else:

            j+=1

    if added>0:

        print "added %s new strings" % (added)

    if removed>0:

        print "removed %s untranslated, obsolete strings" % (removed)

        

    if (added==0 and removed==0):

        print "no strings were changed"

    return messagesOld

    

    

def WriteStrings(messages,language):

    impl = getDOMImplementation()

    folder = os.path.join(OUTPUTPATH,language)

        

    if os.path.exists(folder)==False:

        os.makedirs(folder)

    

    newdoc = impl.createDocument(None, "resources", None)

    resources = newdoc.documentElement

   

    for message in messages:

        string = newdoc.createElement('string')
        
        #The names in strings.xml must be like variables, only alphanumeric and no spaces
        #This regular expression replaces anything that is not alphanumeric, and if it starts with a number, that number is also replaced
        name = re.sub('^[0-9]|[^a-zA-Z0-9]','_',message['source'])

        string.setAttribute('name',name)

        

        t = message['translation'].encode('utf-8')

        #Check translation, if not one, use english source instead

        if t == '' or language == 'en':

            t = message['source']

        
        #Precede apostrophies with \
        pattern = re.compile(r'\'') 
        newT = re.sub(pattern,r"\'",t)
        if newT.count(r"\\")==0:
        	t = newT


        string.appendChild(newdoc.createTextNode(t))

                

        resources.appendChild(string)

    

    with open(os.path.join(folder,'strings.xml'),'w') as f:

        newdoc.writexml(f)

        print 'Wrote %s' % os.path.relpath(f.name,PATH)

        

        

if __name__=="__main__":

    #if len(sys.argv)>1:

    #    path=''

    #    for i in range(1,len(sys.argv)):

    #        path += ' ' + sys.argv[i]

    #    path = path.lstrip(' ')

    #else:

    #    path = os.getcwd()

    OUTPUTPATH = "../i18n"

    PATH = os.getcwd()

    

    

    if os.path.exists(OUTPUTPATH)==False:

        os.makedirs(OUTPUTPATH)

        

        

    files = FindJS(PATH)

    tsFiles = GenerateTs(files)

    GenerateStrings(tsFiles)

    